//
//  MLYTracker.m
//  minilytic iOS SDK
//
//  Copyright (c) 2013 Manbolo. All rights reserved.
//
#import <zlib.h>
#import "MLYApp.h"
#import "MLYDevice.h"
#import "MLYHTTPRequest.h"
#import "MLYTracker.h"
#import "MLYUser.h"

#if !__has_feature(objc_arc)
#pragma message("minilytic iOS SDK use ARC. If your project is not ARC (it's time to change), just use -fobjc-arc compile flags on MLY*.m files.")
#endif


@interface MLYTracker ()

@property(strong) NSMutableArray *trackedItems;
@property(strong) NSDateFormatter *dateFormatter;
@property(strong) NSOperationQueue *requestsQueue;

@end



@implementation MLYTracker

#pragma mark - alloc / init

+ (MLYTracker *)defaultTracker {
    static dispatch_once_t onceToken;
    static MLYTracker *tracker = nil;
    dispatch_once(&onceToken, ^{
        tracker = [[MLYTracker alloc] init];
    });
    return tracker;
}

- (id)init
{
    self = [super init];
    if (self){
        _trackedItems = [NSMutableArray array];
        _dateFormatter = [[NSDateFormatter alloc] init];
        [_dateFormatter setDateFormat:@"yyyy-MM-dd hh:mm:ssZ"];
        _dateFormatter.timeZone = [NSTimeZone timeZoneForSecondsFromGMT:0];
        _dateFormatter.calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
        _dateFormatter.locale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
        _requestsQueue = [[NSOperationQueue alloc] init];
        [self registerToNotifications];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - Background / Foreground
- (void)registerToNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationWillEnterForeground:)
                                                 name:UIApplicationWillEnterForegroundNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationDidEnterBackground:)
                                                 name:UIApplicationDidEnterBackgroundNotification
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationWillTerminate:)
                                                 name:UIApplicationWillTerminateNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationDidFinishLaunching:)
                                                 name:UIApplicationDidFinishLaunchingNotification
                                               object:nil];

}


- (void)applicationWillEnterForeground:(NSNotification *)theNotification
{
    [self trackEvent:@"app.foreground"];
}

- (void)applicationDidEnterBackground:(NSNotification *)theNotification
{
    [self trackEvent:@"app.background"];
    [self sendTrackedItems];
}

- (void)applicationWillTerminate:(NSNotification *)theNotification
{
    [self trackEvent:@"app.background"];
    [self sendTrackedItems];
}

- (void)applicationDidFinishLaunching:(NSNotification *)theNotification
{
    [self trackEvent:@"app.foreground"];
}

#pragma mark - Track event / page
- (void)trackEvent:(NSString *)eventName
{
    NSDictionary *event = [self eventWithName:eventName];
    [_trackedItems addObject:event];
    //NSLog(@"trackEvent: %@", event);
}


- (NSDictionary *)eventWithName:(NSString *)eventName
{
    NSDate *now = [[NSDate alloc] init];
    NSString *date = [_dateFormatter stringFromDate:now];
    return @{@"type": @"event",
             @"name": eventName,
             @"date": date,
             };
}

- (NSData *)JSONDataWithTrackedItems
{
    NSString *appVersion = [MLYApp defaultApp].version;
    NSString *userIdentifier = [MLYUser defaultUser].identifier;
    NSString *devicePlatform = [MLYDevice defaultDevice].platform;
    NSString *deviceHardwareModel = [MLYDevice defaultDevice].hardwareModel;
    NSString *systemVersion = [MLYDevice defaultDevice].systemVersion;
    NSString *systemName = [MLYDevice defaultDevice].systemName;
    NSString *language = [MLYDevice defaultDevice].language;
    
    NSMutableDictionary *payloadDictionary = [NSMutableDictionary dictionary];
    payloadDictionary[@"version"] = @"1";
    if (_accountKey) payloadDictionary[@"account.key"] = _accountKey;
    if (_appIdentifier) payloadDictionary[@"app.identifier"] = _appIdentifier;
    if (appVersion) payloadDictionary[@"app.version"] = appVersion;
    if (userIdentifier) payloadDictionary[@"user.identifier"] = userIdentifier;
    if (devicePlatform) payloadDictionary[@"device.platform"] = devicePlatform;
    if (deviceHardwareModel) payloadDictionary[@"device.hardware_model"] = deviceHardwareModel;
    if (systemVersion) payloadDictionary[@"device.system_version"] = systemVersion;
    if (systemName) payloadDictionary[@"device.system_name"] = systemName;
    if (_trackedItems) payloadDictionary[@"items"] = _trackedItems;
    if (language) payloadDictionary[@"device.language"] = language;

    NSError *writeError = nil;
    NSData  *jsonData = [NSJSONSerialization dataWithJSONObject:payloadDictionary
                                                        options:NSJSONWritingPrettyPrinted
                                                          error:&writeError];
    if (!writeError){
        return jsonData;
    }
    else{
        return nil;
    }
}

- (NSString *)JSONStringWithTrackedItems
{
    NSData *jsonData = [self JSONDataWithTrackedItems];
    NSString *jsonString = [[NSString alloc] initWithData:jsonData
                                                 encoding:NSUTF8StringEncoding];
    return jsonString;
}


- (void)sendTrackedItems
{
    NSData *jsonData = [self JSONDataWithTrackedItems];
    NSData *gzipData = [self GZIPDataWithData:jsonData];
    //NSString *jsonString = [self JSONStringWithTrackedItems];
    //NSLog(@"%@", jsonString);
    
    MLYHTTPRequest *request = [[MLYHTTPRequest alloc] init];
    request.body = gzipData;
    [_requestsQueue addOperation:request];
    [_trackedItems removeAllObjects];
    
}

- (NSData*)GZIPDataWithData:(NSData *)uncompressedData
{
    if ([uncompressedData length] == 0) {
        return uncompressedData;
    }
    
    z_stream zStream;
    bzero(&zStream, sizeof(z_stream));
    
    zStream.zalloc = Z_NULL;
    zStream.zfree = Z_NULL;
    zStream.opaque = Z_NULL;
    zStream.next_in = (Bytef *)[uncompressedData bytes];
    zStream.avail_in = (unsigned int)[uncompressedData length];
    zStream.total_out = 0;
    
    OSStatus status;
    if ((status = deflateInit(&zStream, Z_DEFAULT_COMPRESSION)) != Z_OK) {
        return nil;
    }
    
    NSMutableData *compressedData = [NSMutableData dataWithLength:1024];
    
    do {
        if ((status == Z_BUF_ERROR) || (zStream.total_out == [compressedData length])) {
            [compressedData increaseLengthBy:1024];
        }
        
        zStream.next_out = [compressedData mutableBytes] + zStream.total_out;
        zStream.avail_out = (unsigned int)([compressedData length] - zStream.total_out);
        
        status = deflate(&zStream, Z_FINISH);
    } while ((status == Z_OK) || (status == Z_BUF_ERROR));
    
    deflateEnd(&zStream);
    
    if ((status != Z_OK) && (status != Z_STREAM_END)) {
        return nil;
    }
    
    [compressedData setLength:zStream.total_out];
    
    return compressedData;
}


@end
